/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 360 (Exercise B.2.9)
**
*/

new;
library rpb,pgraph;

cls;

__output = 0;

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

data[.,2:4] = Fill_Miss(data[.,2:4],4);
Dates = data[.,1];

Begin_Date = 20090630;
End_Date = 20120630;
data = selif(data[.,2:4],(Dates .>= Begin_Date) .and (Dates .<= End_Date));

dt = 5;
R = data./lagn(data,dt) - 1.0;
R = trimr(R,dt,0);
nT = rows(R);
n = cols(R);

alpha = 0.95;

/*
**> Question 1
**
*/

print;
print "=====================  Question 1.a  =======================================";

x = ones(n,1)/n;
{VaR_CF2,RC_CF2,databuf} = Compute_Cornish_Fisher_VaR(x,R,alpha);

M1 = vread(databuf,"M1");
M2 = vread(databuf,"M2");
M3 = vread(databuf,"M3");
M4 = vread(databuf,"M4");

let M1[3,1] = 
         22.5124 
          9.9389 
          8.7444 ;
M1 = M1 * 1e-4;

let M2[3,3] = 
          7.0343           0.2931           4.5068 
          0.2931           0.7641           0.3564 
          4.5068           0.3564           6.2416 ;
M2 = M2 * 1e-4;

let M3[3,9] = 
         -6.5540           0.8548          -4.3059           0.8548          -0.2178           0.3454          -4.3059           0.3454          -3.6789 
          0.8548          -0.2178           0.3454          -0.2178          -0.0179          -0.1675           0.3454          -0.1675           0.2505 
         -4.3059           0.3454          -3.6789           0.3454          -0.1675           0.2505          -3.6789           0.2505          -3.8670 ;
M3 = M3 * 1e-6;

let M4[3,27] = 
           2.099            0.022            1.202            0.022            0.049            0.023            1.202            0.023            0.959            0.022            0.049            0.023            0.049            0.006            0.032            0.023            0.032            0.040            1.202            0.023            0.959            0.023            0.032            0.040            0.959            0.040            0.985 
           0.022            0.049            0.023            0.049            0.006            0.032            0.023            0.032            0.040            0.049            0.006            0.032            0.006            0.017            0.007            0.032            0.007            0.050            0.023            0.032            0.040            0.032            0.007            0.050            0.040            0.050            0.092 
           1.202            0.023            0.959            0.023            0.032            0.040            0.959            0.040            0.985            0.023            0.032            0.040            0.032            0.007            0.050            0.040            0.050            0.092            0.959            0.040            0.985            0.040            0.050            0.092            0.985            0.092            1.531 ;
M4 = M4 * 1e-6;

print ftosa(1e4*M1,"%lf",3,4);
print ftosa(1e4*M2,"%lf",3,4);
print ftosa(1e6*M3,"%lf",3,4);
print ftosa(1e6*M4,"%lf",3,3);
print "------------------------------";

mu1 = M1;
mu2 = meanc(R);
print "mean";
print ftosa(100*(mu1~mu2),"%lf",3,3);

sigma1 = sqrt(diag(M2));
sigma2 = stdc(R);
print "standard deviation";
print ftosa(100*(sigma1~sigma2),"%lf",3,3);

sigma = sqrt(diag(M2));

sk1 = (M3[1,1]|M3[2,5]|M3[3,9]) ./ (sigma^3);
sk2 = Estimate_Skewness(R);
print "skewness";
print ftosa((sk1~sk2),"%lf",3,3);

kt1 = (M4[1,1]|M4[2,14]|M4[3,27]) ./ (sigma^4) - 3;
kt2 = Estimate_Kurtosis(R) - 3;
print "excess kurtosis";
print ftosa((kt1~kt2),"%lf",3,3);

print;
print "=====================  Question 1.b  =======================================";

mu1 = x'M1;
mu2 = x'M2*x;
x2 = x .*. x;
mu3 = x'M3*x2;
x3 = x2 .*. x;
mu4 = x'M4*x3;

print;

print ftosa(1e4*mu1~1e4*mu2~1e6*mu3~1e6*mu4,"%lf",3,3);

print;
print "=====================  Question 1.c  =======================================";

mu_L = -mu1;
sigma_L = sqrt(mu2);
gamma1_L = -mu3 / (mu2^1.5);
gamma2_L = mu4 / (mu2^2) - 3;

print ftosa(100*mu_L~100*sigma_L~gamma1_L~gamma2_L,"%lf",3,3);

print;
print "=====================  Question 1.d  =======================================";

z_alpha = cdfni(alpha);
VaR = mu_L + z_alpha .* sigma_L;
mr = -M1 + z_alpha * (M2*x)/sigma_L;
rc = x .* mr;
prc = rc / VaR;
Results = x~mr~rc~prc|error(0)~error(0)~VaR~error(0);
print ftosa(100*Results,"%lf",3,3);

print;
print "=====================  Question 1.e  =======================================";

z = z_alpha + (1/6) * ((z_alpha^2)-1) * gamma1_L + 
              (1/24) * ((z_alpha^3)-3*z_alpha) * gamma2_L - 
              (1/36) * (2*(z_alpha^3)-5*z_alpha) * gamma1_L^2;
VaR_CF = mu_L + z .* sigma_L;

print ftosa(z_alpha~z,"%lf",3,3);

print ftosa(100*(VaR~VaR_CF~VaR_CF2),"%lf",3,3);

gamma1 = -mu3 / (mu2^1.5);
gamma2 = mu4 / (mu2^2) - 3;

d_gamma1 = 3 * mu3 .* (M2 * x) / (mu2^2.5) - 3 * (M3 * x2) ./ (mu2^1.5);
d_gamma2 = 4 * (M4 * x3) ./ (mu2^2) - 4 * mu4 .* (M2 * x) / (mu2^3);
d_z = (1/6) * ((z_alpha^2)-1) * d_gamma1 + (1/24) * ((z_alpha^3)-3*z_alpha) * d_gamma2 - 
      (1/18) * (2*(z_alpha^3)-5*z_alpha) * gamma1 .* d_gamma1;
mr_CF = (-M1 + z * (M2*x)/sqrt(mu2) + d_z * sqrt(mu2) );
rc_CF = x .* mr_CF;
prc_CF = rc_CF / VaR_CF;

Results = x~mr_CF~rc_CF~prc_CF|error(0)~error(0)~VaR_CF~error(0);
print ftosa(100*Results,"%lf",3,3);

print ftosa(100*(rc_CF~rc_CF2),"%lf",3,3);

/*
**> Question 2
**
*/

print;
print "=====================  Question 2.a  =======================================";

x0 = ones(n,1)/n;
{x,VaR,mr,rc,prc} = Compute_ERC_VaR(x0,M1,M2,alpha);

Results = x~mr~rc~prc|error(0)~error(0)~VaR~error(0);
print ftosa(100*Results,"%lf",3,3);

mu1 = x'M1;
mu2 = x'M2*x;
mu_L = -mu1;
sigma_L = sqrt(mu2);

z_alpha = cdfni(alpha);
VaR = mu_L + z_alpha .* sigma_L;
mr = -M1 + z_alpha * (M2*x)/sigma_L;
rc = x .* mr;
prc = rc / VaR;

Results = x~mr~rc~prc|error(0)~error(0)~VaR~error(0);
print ftosa(100*Results,"%lf",3,3);

print;
print "=====================  Question 2.b  =======================================";

x0 = ones(n,1)/n;
b = x0;
_rpb_rb_mtd = 1;

{x,VaR,mr,rc,pc} = Compute_RB_CF(x0,M1,M2,M3,M4,b,alpha);
Results = x~mr~rc~pc|error(0)~error(0)~VaR~error(0);
print ftosa(100*Results,"%lf",3,3);

print;
print "=====================  Question 2.c  =======================================";

alpha = 0.99;
x0 = ones(n,1)/n;
{x,VaR,mr,rc,prc} = Compute_ERC_VaR(x0,M1,M2,alpha);

Results = x~mr~rc~prc|error(0)~error(0)~VaR~error(0);
print ftosa(100*Results,"%lf",3,3);

{x,VaR,mr,rc,prc} = Compute_RB_CF(x0,M1,M2,M3,M4,b,alpha);
Results = x~mr~rc~prc|error(0)~error(0)~VaR~error(0);
print ftosa(100*Results,"%lf",3,3);
