/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 60 (Table 1.13)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';

sigma_target = 0.15;
let x0 = 0.30 0.45 0.15 0.10;
sigma0 = sqrt(x0'cov*x0);
mu0 = x0'mu;

print ftosa(100*mu0,"mu0 = %lf",3,2);
print ftosa(100*sigma0,"sigma0 = %lf",3,2);
print "-------------------------------------------------------------";

A = ones(1,n); B = 1;
Bounds = (0~1) .* ones(n,1);

{x1,mu1,sigma1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,sigma_target,2);
tau1 = sumc(abs(x1-x0)); 

print ftosa(100*x1,"%lf",3,2);
print ftosa(100*mu1,"mu1 = %lf",3,2);
print ftosa(100*sigma1,"sigma1 = %lf",3,2); 
print ftosa(100*tau1,"tau1 = %lf",3,2); 
print "-------------------------------------------------------------";

z0 = zeros(n,n);
cov_tilde = cov~z0~z0 |
            z0~z0~z0 |
            z0~z0~z0 ;

z0 = zeros(n,1);
mu_tilde = mu|z0|z0;

z0 = zeros(1,n);
z1 = eye(n);
A_tilde = ones(1,n)~z0~z0 |
          z1~z1~-z1; 
B_tilde = 1|x0;

z0 = zeros(1,n);
z1 = ones(1,n);

C_tilde = z0~-z1~-z1;
Bounds_tilde = (0~1) .* ones(3*n,1);

let tau_vector = 0.05 0.10 0.25 0.50 0.75;
x2 = zeros(n,5);
mu2 = zeros(5,1);
sigma2 = zeros(5,1);
tau2 = zeros(5,1);

i = 1;
do until i > 5;
  D_tilde = -tau_vector[i];
  {x_tilde,mu2[i],sigma2[i],gamma2,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,C_tilde,D_tilde,Bounds_tilde,sigma_target,2);
  x_tilde = reshape(x_tilde,3,n)'; 
  x2[.,i] = x_tilde[.,1];
  tau2[i] = sumc(vec(x_tilde[.,2 3]));
  i = i + 1;
endo;

print ftosa(100*(0~tau_vector'),"%lf",3,2);
print ftosa(100*(x1~x2),"%lf",3,2);
print ftosa(100*(mu1~mu2'),"%lf",3,2);
print ftosa(100*(sigma1~sigma2'),"%lf",3,2);
print "-------------------------------------------------------------";



