/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 90 (Figure 2.2)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

theta = 2;

mu1 = 0.10;
sigma1 = 0.20;
n1 = 6;
w1 = 0.50;

mu2 = 0.10;
sigma2 = 0.25;
n2 = 4;
w2 = 0.50;

w = w1|w2;

alpha = 0.90;
nS = 5000;
nR = 2000;

m1 = nS;
indx_alpha = m1 * alpha;
h = 2.5/100;
h = h * m1;
indx = seqa(indx_alpha-h,1,2*h+1);
p1 = zeros(m1,1);
p1[indx] = ones(2*h+1,1)/(2*h+1);

m2 = 1000;
indx_alpha = m2 * alpha;
h = 2.5/100;
h = h * m2;
indx = seqa(indx_alpha-h,1,2*h+1);
p2 = zeros(m2,1);
p2[indx] = ones(2*h+1,1)/(2*h+1);

SD = zeros(nR,1);
RC_SD = zeros(2,nR);

VaR = zeros(nR,1);
RC_VaR = zeros(2,nR);
RC2_VaR = zeros(2,nR);

VaR_w = zeros(nR,1);
RC_VaR_w = zeros(2,nR);
RC2_VaR_w = zeros(2,nR);

ES = zeros(nR,1);
RC_ES = zeros(2,nR);
RC2_ES = zeros(2,nR);

ES_w = zeros(nR,1);
RC_ES_w = zeros(2,nR);
RC2_ES_w = zeros(2,nR);

i = 1;
do until i > nR;
  {u1,u2} = rndCopulaClayton(theta,ns);

  R1 = mu1 + sigma1 * cdfti(u1,n1);
  R2 = mu2 + sigma2 * cdfti(u2,n2);

  cov = vcx(R1~R2);
  SD[i] = sqrt(w'cov*w);
  RC_SD[.,i] = w .* (cov*w)/sqrt(w'cov*w);

  L1 = -w1 * R1;
  L2 = -w2 * R2;

  L = L1~L2;

  {VaR[i],RC_VaR[.,i],ES[i],RC_ES[.,i]} = Compute_Empirical_RC(L,alpha);
  {VaR_w[i],RC_VaR_w[.,i],ES_w[i],RC_ES_w[.,i]} = Compute_Empirical_RC(L,p1);

  {VaR[i],RC2_VaR[.,i],ES[i],RC2_ES[.,i]} = Compute_Empirical_RC(L[1:m2,.],alpha);
  {VaR_w[i],RC2_VaR_w[.,i],ES_w[i],RC2_ES_w[.,i]} = Compute_Empirical_RC(L[1:m2,.],p2);

  i = i + 1;
endo;

RC_ES = RC_ES';
RC2_ES = RC2_ES';

RC_VaR = RC_VaR';
RC2_VaR = RC2_VaR';

RC_VaR_w = RC_VaR_w';
RC2_VaR_w = RC2_VaR_w';

x = seqa(-0.30,0.70/1500,1501);
d = zeros(rows(x),6);

{d[.,1],F} = regKernelDensity(RC_ES[.,1],x);
{d[.,2],F} = regKernelDensity(RC2_ES[.,1],x);
x1 = 100*meanc(RC_ES[.,1]);

{d[.,3],F} = regKernelDensity(RC_VaR[.,1],x);
{d[.,4],F} = regKernelDensity(RC2_VaR[.,1],x);
x2 = 100*meanc(RC_VaR[.,1]);

{d[.,5],F} = regKernelDensity(RC_VaR_w[.,1],x);
{d[.,6],F} = regKernelDensity(RC2_VaR_w[.,1],x);
x3 = 100*meanc(RC_VaR_w[.,1]);

graphset;
  window3u;

  _pdate = ""; _pnum = 2; _pframe = 0;
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20; 
  _pltype = 6|1|3; _plwidth = 10; 
  xtics(0,0.50,0.10,2);

  setwind(1);
    title("\216ES");
    xtics(8,16,2,0);
    ytics(0,80,10,0);
    _pcolor = 1;
    _pline = 1~3~x1~0~x1~100~1~0~10;
    _plegstr = "m = 5000\000m = 1000";
    _plegctl = {2 5 9 3};
    xy(100*x,d[.,1 2]);

    _plegctl = 0;

  setwind(2);
    title("\216VaR");
    xtics(-30,40,10,0);
    ytics(0,6,1,0);
    _pcolor = 2;
    _pline = 1~3~x2~0~x2~100~1~0~10;
    xy(100*x,d[.,3 4]);

  setwind(3);
    title("\216VaR (with regularization)");
    xtics(2,10,2,0);
    ytics(0,70,10,0);
    _pcolor = 3;
    _pline = 1~3~x3~0~x3~100~1~0~10;
    xy(100*x,d[.,5 6]);

    graphprt("-c=1 -cf=chap2-1-3-3.eps");

  endwind;    


  