/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 118 & 119 (Tables 2.18 & 2.19)
**
*/

new;

// #define use_co 1

#ifdef use_co
  library rpb,pgraph,co;
#else
  library rpb,pgraph;
#endif

cls;

let sigma = 0.20 0.15 0.25;
n = rows(sigma);

_sigma_ = sigma;

if 1;
  let rho = 1.0
            0.7 1.0
            0.4 0.5 1.0;
  rho = xpnd(rho);
else;
  rho = 0.20;
  rho = diagrv(rho*ones(n,n),1.0);
endif;

Sigma = rho .* sigma .* sigma';
call invpd(Sigma);

A = ones(1,n); B = 1; e = ones(n,1);
bounds = (0.0~1.0) .* ones(n,1);

sv = ones(n,1)/n;
{x_GMV,u1,u2,u3,u4,retcode} = Qprog(sv,Sigma,zeros(n,1),A,B,0,0,0);
{x_tilde,lambda0,u2,lambda_minus,lambda_plus,retcode} = Qprog(sv,Sigma,zeros(n,1),A,B,0,0,Bounds);

Sigma_tilde = Sigma + (lambda_plus - lambda_minus) * e' + e * (lambda_plus - lambda_minus)';
call invpd(Sigma_Tilde);

{x_bar,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde,zeros(n,1),A,B,0,0,0);

results = x_GMV~x_tilde~x_bar;
print ftosa(100*results,"%lf",5,3);

let rb = 0.50 0.30 0.20;

{x0,vol,mr,rc,prc} = Compute_RB_Portfolio(sv,Sigma,rb);
c0 = sumc( rb .* ln(x0) );

sv = x0;

declare matrix _external_c;

proc MinProc1(x);
  retp( sqrt(x'Sigma*x) );
endp;

proc IneqProc1(x);
  retp( sumc( rb .* ln(x)) - _external_c);
endp;

proc MinProc2(x);
  retp( 0.5*x'Sigma*x );
endp;

proc IneqProc2(x);
  retp( sumc( rb .* ln(x)) - _external_c);
endp;

#ifdef use_co

errorlog "co";

_co_A = error(0);
_co_B = error(0);
_co_C = error(0);
_co_D = error(0);
_co_Bounds = Bounds;

_external_c = c0;
_co_IneqProc = &IneqProc1;
{x1,fmin,gdr,retcode} = co(&MinProc1,sv);
lambda0 = vread(_co_Lagrange,"lineq");
lambda_c1 = vread(_co_Lagrange,"nlinineq");
vol1 = sqrt(x1'*Sigma*x1);
foc1 = (Sigma*x1)/vol1 - lambda_c1 .* rb ./ x1;

_external_c = c0;
_co_IneqProc = &IneqProc2;
{x2,fmin,gdr,retcode} = co(&MinProc2,sv);
lambda0 = vread(_co_Lagrange,"lineq");
lambda_c2 = vread(_co_Lagrange,"nlinineq");              // lambda_c2 = lambda_c1 * vol1
vol2 = sqrt(x2'*Sigma*x2);
foc2 = Sigma*x2 - lambda_c2 .* rb ./ x2;

#else

errorlog "sqp";

_sqp_A = error(0);
_sqp_B = error(0);
_sqp_C = error(0);
_sqp_D = error(0);
_sqp_Bounds = Bounds;

_external_c = c0;
_sqp_IneqProc = &IneqProc1;
{x1,fmin,lagr,retcode} = sqpSolve(&MinProc1,sv);
lambda0 = vread(lagr,"lineq");
lambda_c1 = vread(lagr,"nlinineq");
vol1 = sqrt(x1'*Sigma*x1);
foc1 = (Sigma*x1)/vol1 - lambda_c1 .* rb./x1;

_external_c = c0;
_sqp_IneqProc = &IneqProc2;
{x2,fmin,lagr,retcode} = sqpSolve(&MinProc2,sv);
lambda0 = vread(lagr,"lineq");
lambda_c2 = vread(lagr,"nlinineq");
foc2 = Sigma*x2 - lambda_c2 .* rb./x1;

#endif

lambda_c = lambda_c2;
x = x2;
x = x / sumc(x);
Lambda = lambda_c .* rb ./ x;
D = diagrv(eye(n),lambda_c ./ x);

Sigma_tilde1 = Sigma + lambda_c * n * (e*e') - (lambda*e' + e*lambda'); 
Sigma_tilde2 = Sigma + n * D - (Lambda*e' + e*lambda'); 

I = diagrv(ones(n,n),0);

alpha2 = lambda_c * sumc( vec( ((x + x') ./ x ./ x') .* I))/(n^2 - (sumc(1./x)^2)/sumc(1./(x^2)));
alpha1 = 2 - 1/(lambda_c)*sumc(1./x)/sumc(1./(x^2))*alpha2;

Sigma_tilde3 = Sigma + alpha1 * n * D + alpha2 * (1/n)*(e*e') - (Lambda*e' + e*lambda'); 

proc (1) = optimize_Frobenious_4(c);
  local Sigma_Tilde,Q,norm;
  Sigma_Tilde = Sigma + c[1]*D + c[2] * (1/n)*(e*e') - (Lambda*e' + e*lambda'); 
  Q = Sigma_Tilde - Sigma;
  norm = sumc(diag(Q'Q));
  retp(norm);
endp;

{c,fmin,grd,retcode} = Qnewton(&optimize_Frobenious_4,0.5|0.5);
Sigma_Tilde4 = Sigma + c[1]*D + c[2] * (1/n)*(e*e') - (Lambda*e' + e*lambda'); 

proc (1) = optimize_Frobenious_5(c);
  local Sigma_Tilde,Q,norm;
  Sigma_Tilde = c[1] * sigma_tilde1 + c[2] * sigma_tilde2;
  Q = Sigma_Tilde - Sigma;
  norm = sumc(diag(Q'Q));
  retp(norm);
endp;

{c,fmin,grd,retcode} = Qnewton(&optimize_Frobenious_5,0.5|0.5);
Sigma_Tilde5 = c[1] * sigma_tilde1 + c[2] * sigma_tilde2;

lambda_min = minc(abs(eig(Sigma_Tilde1)~eig(Sigma_Tilde2)~eig(Sigma_Tilde3)~eig(Sigma_Tilde4)~eig(Sigma_Tilde5)));

cls;

print ftosa(lambda_min',"%lf",3,5);
print ftosa(Sigma_tilde1*x~Sigma_tilde2*x~Sigma_tilde3*x~Sigma_tilde4*x~Sigma_tilde5*x,"%lf",3,5);

sigma_tilde1 = normalize_vol(sigma,sigma_tilde1);
sigma_tilde2 = normalize_vol(sigma,sigma_tilde2);
sigma_tilde3 = normalize_vol(sigma,sigma_tilde3);
sigma_tilde4 = normalize_vol(sigma,sigma_tilde4);

{x_bar1,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde1,zeros(n,1),A,B,0,0,0);
{x_bar2,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde2,zeros(n,1),A,B,0,0,0);
{x_bar3,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde3,zeros(n,1),A,B,0,0,0);
{x_bar4,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde4,zeros(n,1),A,B,0,0,0);
{x_bar5,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde5,zeros(n,1),A,B,0,0,0);
if retcode /= 0;
  x_bar4 = error(0);
endif;

print ftosa(100*transform(sigma),"%lf",5,3);
print ftosa(100*transform(sigma_tilde1),"%lf",5,2);
print ftosa(100*transform(sigma_tilde2),"%lf",5,2);
print ftosa(100*transform(sigma_tilde3),"%lf",5,2);
print ftosa(100*transform(sigma_tilde4),"%lf",5,2);
print ftosa(100*transform(sigma_tilde5),"%lf",5,2);

results = x0~x_bar1~x_bar2~x_bar3~x_bar4;
print ftosa(100*results,"%lf",5,3);

proc (1) = transform(sigma);
  retp(sqrt(diag(sigma))~corrvc(sigma));
endp;

proc normalize_vol(sigma,sigma_tilde);
  local vol,vol_tilde,alpha;

  vol = diag(sigma);
  vol_tilde = diag(sigma_tilde);
  alpha = meanc(vol)/meanc(vol_tilde);
  retp( alpha * sigma_tilde);
endp;