/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 145 & 146 (Tables 2.33 & 2.34)
**
*/

new;

#define use_co 1

#ifdef use_co
  library rpb,pgraph,co;
#else
  library rpb,pgraph;
#endif

cls;

Let A[4,3] = 0.9    0.0     0.5
             1.1    0.5     0.0
             1.2    0.3     0.2
             0.8    0.1     0.7;

let vol_F = 0.20 0.10 0.10;
Omega = diagrv(eye(3),vol_F^2);
let sigma_A = 0.10 0.15 0.10 0.15;
D = diagrv(eye(4),sigma_A^2);

Sigma = A*Omega*A' + D;
vol_A = sqrt(diag(Sigma));
rho_A = Sigma ./ vol_A ./ vol_A';

n = rows(A);
m = cols(A);

print ftosa(100*vol_A,"%lf",3,2);
print ftosa(100*rho_A,"%lf",3,2);

indx = seqa(1,1,3);

_rpb_factor_bounds = 0;

x = zeros(n,3);
y = zeros(n,3);
f_mr = zeros(n,3);
f_rc = zeros(n,3);
f_prc = zeros(n,3);

i = 1;
do until i > 3;
  {x[.,i],y[.,i],f_mr[.,i],f_rc[.,i],f_prc[.,i]} = Compute_RB_Factor(0,Sigma,A,Omega,D,0,indx,i+1);
  i = i + 1;
endo;

cls;

i = 1;
do until i > 3;
  {vol,mr,rc,prc} = Compute_Risk_Contribution(x[.,i],Sigma);
  p = f_rc[1:m,i];
  p = p/sumc(p);
  {H,H_star,N_eff} = Herfindahl_Index(p);
  {G1,G2} = Gini_Index(p);
  {I1,I2} = Entropy_Index(p);

  print "------------------------------------------------------";
  print ftos(100*vol,"vol = %lf",3,2);
  print ftosa(100*(y[.,i]~f_mr[.,i]~f_rc[.,i]~f_prc[.,i]),"%lf",3,2);
  print ftos(100*vol,"vol = %lf",3,2);
  print ftosa(100*(x[.,i]~mr~rc~prc),"%lf",3,2);
  print;
  print ftosa(100*(H_star~G1~I2),"%lf",3,2);
  
  i = i + 1;
endo;
