/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 182 (Table 3.12)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data//";
datFileName = Pathname $+ "sx5e-info";
loads info = ^datFileName;
datFileName = Pathname $+ "sx5e-weights";
Composition = loadd(datFileName);
datFileName = Pathname $+ "sx5e-prices";
Prices = loadd(datFileName);
nStocks = cols(Prices) - 2;

Weights = Composition[.,2:cols(Composition)]/100;

Rebalancing_Dates = Composition[.,1];
Rebalancing_Dates[1] = 19920101;
nRD = rows(Rebalancing_Dates);

Dates = Prices[.,1];
Index = Prices[.,2];
Prices = Prices[.,3:2+nStocks];

EndDate = 20120928;

Lag_CovMatrix = 260;
FileName = itos("Results\\sx5e-Weights-basic-",Lag_CovMatrix);
loadm Computed_Weights = ^FileName;

Weights_MV = array3d_get_x(Computed_Weights,1);
Weights_ERC = array3d_get_x(Computed_Weights,2);
Weights_MDP = array3d_get_x(Computed_Weights,3);
Weights_EW = array3d_get_x(Computed_Weights,4);

Index_MV = Simulate_Backtest(Rebalancing_Dates,Weights_MV,Dates,Prices,0);
Index_ERC = Simulate_Backtest(Rebalancing_Dates,Weights_ERC,Dates,Prices,0);
Index_MDP = Simulate_Backtest(Rebalancing_Dates,Weights_MDP,Dates,Prices,0);
Index_EW = Simulate_Backtest(Rebalancing_Dates,Weights_EW,Dates,Prices,0);

Index_MDP[4391] = 0.5*(Index_MDP[4390]+Index_MDP[4392]);

Data = Dates~Index~Index_MV~Index_ERC~Index_MDP~Index_EW;
Data = packr(Data);
Data = selif(Data, (Data[.,1] .>= 19921231) .and (Data[.,1] .<= EndDate));

Dates = Data[.,1];
nDates = rows(Dates);
Data[.,2] = Data[.,2] ./ Data[1,2] * 1;
Data = 100* Data[.,2:6];

nYears = 2012-1992+1;
Years = zeros(nYears,cols(data));
Perf1Y = zeros(nYears,cols(data));
Vol1Y = zeros(nYears,cols(data));

i = 1;
do until i > cols(data);
  {Years[.,i],Perf1Y[.,i],Vol1Y[.,i]} = Estimate_Yearly_Statistics(Dates,Data[.,i],0,0,0);
  i = i + 1;
endo;

Results = trimr(Years[.,1]~100*Perf1Y[.,1 5 2 4 3],1,0);
print ftosa(Results,"%lf",3,1);