/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 290 & 291 (Tables 6.6 & 6.7)
**
*/

new;
library rpb,pgraph;

cls;

datPathName = "Data\\";
datFileName = datPathName $+ "rfp-saa";

Data = loadd(datFileName);
Data = selif(Data,Data[.,1] .>= 199901);
Dates = Data[.,1];
nDates = rows(Dates);

vnames = "" $+ getname(datFileName);

let string Asset_Classes[13,1] = EQY_USD EQY_EUR EQY_GBP EQY_JPY
                                 SOV_USD SOV_EUR SOV_GBP SOV_JPY
                                 BND_USD BND_EUR
                                 HY_USD HY_EUR
                                 TIPS_USD;

let string Risk_Classes[7,1] =  GDP IP CPI YC RR FXE COMM;
let B[4,7] = 1 1 0 0 0 0 0
             0 0 1 0 0 0 1
             0 0 0 1 1 0 0
             0 0 0 0 0 1 0;

n = rows(Asset_Classes);
m = rows(Risk_Classes);
A = zeros(n,m);
Omega = zeros(m,m);
D = zeros(n,n);
R_Assets = zeros(nDates,n);

i = 1;
do until i > n;
  indx_Assets = indsav(Asset_Classes[i],vnames);
  Currency = strsect(Asset_Classes[i],strlen(Asset_Classes[i])-2,3);
  Risk_Factors = Risk_Classes $+ "_" $+ Currency;
  indx_Factors = indsav(Risk_Factors,vnames);
  y = Data[.,indx_Assets];
  x = Data[.,indx_Factors];
  beta_ = y/x;
  A[i,.] = beta_';
  R_Assets[.,i] = y;
  if i == 1;
    R_Factors = x;
  endif;
  i = i + 1;
endo;

cov = vcx(R_Assets);
Omega = vcx(R_Factors);
D = cov - A*Omega*A';
D = diagrv(zeros(n,n),diag(D));
Sigma = A*Omega*A' + D;

cls;

let x = 0.20 0.20 0.05 0.05
        0.10 0.05 0.05 0.05
        0.05 0.05
        0.05 0.05
        0.05;
print sumc(x);
{vol,y,mr,rc,prc} = Compute_RC_Factor(x,Sigma,A,Omega,D);
prc1 = B*prc[1:m]|sumc(prc[m+1:n]);

sv = x;
rb = prc;

let x = 0.10 0.10 0.00 0.00
        0.20 0.15 0.05 0.05
        0.05 0.05
        0.05 0.05
        0.15;
print sumc(x);
{vol,y,mr,rc,prc} = Compute_RC_Factor(x,Sigma,A,Omega,D);
prc2 = B*prc[1:m]|sumc(prc[m+1:n]);

let x = 0.30 0.30 0.10 0.10
        0.10 0.10 0.00 0.00
        0.00 0.00
        0.00 0.00
        0.00;
print sumc(x);
{vol,y,mr,rc,prc} = Compute_RC_Factor(x,Sigma,A,Omega,D);
prc3 = B*prc[1:m]|sumc(prc[m+1:n]);

let rb = 0.33 0.05 0.02 0.22 0.22 0.07 0.10;
let rb = 0.30 0.04 0.05 0.20 0.20 0.05 0.15;
// rb = rb /  sumc(rb);
print rb;

{x,y,mr,rc,prc} = Compute_RB_Factor(x,Sigma,A,Omega,D,rb,seqa(1,1,m),0);
prc4 = B*prc[1:m]|sumc(prc[m+1:n]);
prc5 = B*rb|0;

cls;

prc = prc1~prc2~prc3~prc4~prc5;
print "Risk contributions = ";
print ftosa(100*prc,"%lf",3,2);

print "Solution = ";
x = x .* dotfne(x,0);
print ftosa(100*miss(x,0),"%lf",3,2);
